from http import HTTPStatus
from typing import Any, Optional, Union

import httpx
from pydantic import Field

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from ...models.add_sheet_request import AddSheetRequest
from ...models.add_sheet_response import AddSheetResponse
from ...models.default_error import DefaultError


def _get_kwargs(
    *,
    body: AddSheetRequest,
    spreadsheet: str = Field(alias="spreadsheet"),
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    params["spreadsheet"] = spreadsheet

    params = {k: v for k, v in params.items() if v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/AddSheet",
        "params": params,
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[AddSheetResponse, DefaultError]]:
    if response.status_code == 200:
        response_200 = AddSheetResponse.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = DefaultError.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = DefaultError.from_dict(response.json())

        return response_401
    if response.status_code == 403:
        response_403 = DefaultError.from_dict(response.json())

        return response_403
    if response.status_code == 404:
        response_404 = DefaultError.from_dict(response.json())

        return response_404
    if response.status_code == 405:
        response_405 = DefaultError.from_dict(response.json())

        return response_405
    if response.status_code == 406:
        response_406 = DefaultError.from_dict(response.json())

        return response_406
    if response.status_code == 409:
        response_409 = DefaultError.from_dict(response.json())

        return response_409
    if response.status_code == 415:
        response_415 = DefaultError.from_dict(response.json())

        return response_415
    if response.status_code == 500:
        response_500 = DefaultError.from_dict(response.json())

        return response_500
    if response.status_code == 402:
        response_402 = DefaultError.from_dict(response.json())

        return response_402
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[AddSheetResponse, DefaultError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: AddSheetRequest,
    spreadsheet: str = Field(alias="spreadsheet"),
    spreadsheet_lookup: Any,
) -> Response[Union[AddSheetResponse, DefaultError]]:
    """Add sheet

     Adds a new sheet to the specified spreadsheet

    Args:
        spreadsheet (str):
        body (AddSheetRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[AddSheetResponse, DefaultError]]
    """

    if not spreadsheet and spreadsheet_lookup:
        lookup_response_raw = client.get_httpx_client().request(
            method="get", url="/EventFolderPicker"
        )
        lookup_response = lookup_response_raw.json()

        found_items = []
        for item in lookup_response:
            if spreadsheet_lookup in item["FullName"]:
                found_items.append(item)

        if not found_items:
            raise ValueError(
                "No matches found for spreadsheet_lookup in EventFolderPicker"
            )
        if len(found_items) > 1:
            print(
                "Warning: Multiple matches found for spreadsheet_lookup in EventFolderPicker. Using the first match."
            )

        spreadsheet = found_items[0]["ID"]

    kwargs = _get_kwargs(
        body=body,
        spreadsheet=spreadsheet,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: AddSheetRequest,
    spreadsheet: str = Field(alias="spreadsheet"),
    spreadsheet_lookup: Any,
) -> Optional[Union[AddSheetResponse, DefaultError]]:
    """Add sheet

     Adds a new sheet to the specified spreadsheet

    Args:
        spreadsheet (str):
        body (AddSheetRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[AddSheetResponse, DefaultError]
    """

    return sync_detailed(
        client=client,
        body=body,
        spreadsheet=spreadsheet,
        spreadsheet_lookup=spreadsheet_lookup,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: AddSheetRequest,
    spreadsheet: str = Field(alias="spreadsheet"),
    spreadsheet_lookup: Any,
) -> Response[Union[AddSheetResponse, DefaultError]]:
    """Add sheet

     Adds a new sheet to the specified spreadsheet

    Args:
        spreadsheet (str):
        body (AddSheetRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[AddSheetResponse, DefaultError]]
    """

    if not spreadsheet and spreadsheet_lookup:
        lookup_response_raw = await client.get_async_httpx_client().request(
            method="get", url="/EventFolderPicker"
        )
        lookup_response = lookup_response_raw.json()

        found_items = []
        for item in lookup_response:
            if spreadsheet_lookup in item["FullName"]:
                found_items.append(item)

        if not found_items:
            raise ValueError(
                "No matches found for spreadsheet_lookup in EventFolderPicker"
            )
        if len(found_items) > 1:
            print(
                "Warning: Multiple matches found for spreadsheet_lookup in EventFolderPicker. Using the first match."
            )

        spreadsheet = found_items[0]["ID"]

    kwargs = _get_kwargs(
        body=body,
        spreadsheet=spreadsheet,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: AddSheetRequest,
    spreadsheet: str = Field(alias="spreadsheet"),
    spreadsheet_lookup: Any,
) -> Optional[Union[AddSheetResponse, DefaultError]]:
    """Add sheet

     Adds a new sheet to the specified spreadsheet

    Args:
        spreadsheet (str):
        body (AddSheetRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[AddSheetResponse, DefaultError]
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
            spreadsheet=spreadsheet,
            spreadsheet_lookup=spreadsheet_lookup,
        )
    ).parsed
